﻿using System.Collections.Generic;
using System.Linq;
using System.Windows.Media;

namespace ElectroKnob
{


    /// <summary>
    /// Color Description
    /// </summary>
    public class ColorDescriptor
    {
        /// <summary>
        /// Gets or sets the name of the color.
        /// </summary>
        /// <value>The name of the color.</value>
        public string ColorName { get; set; }
        /// <summary>
        /// Gets or sets the color code.
        /// </summary>
        /// <value>The color code.</value>
        public uint ColorCode { get; set; }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return ColorName;
        }

        /// <summary>
        /// Performs an implicit conversion from <see cref="ElectroKnob.ColorDescriptor"/> to <see cref="System.Windows.Media.Color"/>.
        /// </summary>
        /// <param name="color">The color.</param>
        /// <returns>The result of the conversion.</returns>
        public static implicit operator Color(ColorDescriptor color)
        {
            var value = color.ColorCode;

            return Color.FromArgb(
                (byte)(value >> 24),
                (byte)(value >> 16),
                (byte)(value >> 8),
                (byte)value
                );
        }

        /// <summary>
        /// Performs an implicit conversion from <see cref="ElectroKnob.ColorDescriptor"/> to <see cref="System.Windows.Media.Brush"/>.
        /// </summary>
        /// <param name="color">The color.</param>
        /// <returns>The result of the conversion.</returns>
        public static implicit operator Brush(ColorDescriptor color)
        {
            return new SolidColorBrush(color);
        }

        /// <summary>
        /// Performs an implicit conversion from <see cref="ElectroKnob.ColorDescriptor"/> to <see cref="System.UInt32"/>.
        /// </summary>
        /// <param name="color">The color.</param>
        /// <returns>The result of the conversion.</returns>
        public static implicit operator uint(ColorDescriptor color)
        {
            return color.ColorCode;
        }

        /// <summary>
        /// Equalses the specified other.
        /// </summary>
        /// <param name="other">The other.</param>
        /// <returns></returns>
        public bool Equals(ColorDescriptor other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return Equals(other.ColorName, ColorName) && other.ColorCode == ColorCode;
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> to compare with the current <see cref="T:System.Object"/>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>; otherwise, false.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">The <paramref name="obj"/> parameter is null.</exception>
        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != typeof(ColorDescriptor)) return false;
            return Equals((ColorDescriptor)obj);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"/>.
        /// </returns>
        public override int GetHashCode()
        {
            unchecked
            {
                return ((ColorName != null ? ColorName.GetHashCode() : 0) * 397) ^ ColorCode.GetHashCode();
            }
        }
    }

    /// <summary>
    /// Defines all WPF colors for Silverlight.
    /// Original idea was found on Silverlight Forums.
    /// Modified by O.Dahan, 2009. odahan@gmail.com. http://www.e-naxos.com/blog
    /// </summary>
    public static class AllColors
    {
        static AllColors()
        {
            PredefinedColors = new List<ColorDescriptor>
      {
         new ColorDescriptor{ColorName = "Aliceblue", ColorCode = 0xFFF0F8FF},
         new ColorDescriptor{ColorName = "AntiqueWhite", ColorCode = 0xFFFAEBD7},
         new ColorDescriptor{ColorName = "Aqua", ColorCode = 0xFF00FFFF},
         new ColorDescriptor{ColorName = "Aquamarine", ColorCode = 0xFF7FFFD4},
         new ColorDescriptor{ColorName = "Azure", ColorCode = 0xFF5291EF},
         new ColorDescriptor{ColorName = "Beige", ColorCode = 0xFFF5F5DC},
         new ColorDescriptor{ColorName = "Bisque", ColorCode = 0xFFFFE4C4},
         new ColorDescriptor{ColorName = "Black", ColorCode = 0xFF000000},
         new ColorDescriptor{ColorName = "BlanchedAlmond", ColorCode = 0xFFFFEBCD},
         new ColorDescriptor{ColorName = "Blue", ColorCode = 0xFF0000FF},
         new ColorDescriptor{ColorName = "BlueViolet", ColorCode = 0xFF8A2BE2},
         new ColorDescriptor{ColorName = "Brown", ColorCode = 0xFFA52A2A},
         new ColorDescriptor{ColorName = "BurlyWood", ColorCode = 0xFFDEB887},
         new ColorDescriptor{ColorName = "CadetBlue", ColorCode = 0xFF5F9EA0},
         new ColorDescriptor{ColorName = "Chartreuse", ColorCode = 0xFF7FFF00},
         new ColorDescriptor{ColorName = "Chocolate", ColorCode = 0xFFD2691E},
         new ColorDescriptor{ColorName = "Coral", ColorCode = 0xFFFF7F50},
         new ColorDescriptor{ColorName = "CornflowerBlue", ColorCode = 0xFF6495ED},
         new ColorDescriptor{ColorName = "Cornsilk", ColorCode = 0xFFFFF8DC},
         new ColorDescriptor{ColorName = "Crimson", ColorCode = 0xFFDC143C},
         new ColorDescriptor{ColorName = "Cyan", ColorCode = 0xFF00FFFF},
         new ColorDescriptor{ColorName = "DarkBlue", ColorCode = 0xFF00008B},
         new ColorDescriptor{ColorName = "DarkCyan", ColorCode = 0xFF008B8B},
         new ColorDescriptor{ColorName = "DarkGoldenrod", ColorCode = 0xFFB8860B},
         new ColorDescriptor{ColorName = "DarkGray", ColorCode = 0xFFA9A9A9},
         new ColorDescriptor{ColorName = "DarkGreen", ColorCode = 0xFF006400},
         new ColorDescriptor{ColorName = "DarkKhaki", ColorCode = 0xFFBDB76B},
         new ColorDescriptor{ColorName = "DarkMagenta", ColorCode = 0xFF8B008B},
         new ColorDescriptor{ColorName = "DarkOliveGreen", ColorCode = 0xFF556B2F},
         new ColorDescriptor{ColorName = "DarkOrange", ColorCode = 0xFFFF8C00},
         new ColorDescriptor{ColorName = "DarkOrchid", ColorCode = 0xFF9932CC},
         new ColorDescriptor{ColorName = "DarkRed", ColorCode = 0xFF8B0000},
         new ColorDescriptor{ColorName = "DarkSalmon", ColorCode = 0xFFE9967A},
         new ColorDescriptor{ColorName = "DarkSeaGreen", ColorCode = 0xFF8FBC8F},
         new ColorDescriptor{ColorName = "DarkSlateBlue", ColorCode = 0xFF483D8B},
         new ColorDescriptor{ColorName = "DarkSlateGray", ColorCode = 0xFF2F4F4F},
         new ColorDescriptor{ColorName = "DarkTurquoise", ColorCode = 0xFF00CED1},
         new ColorDescriptor{ColorName = "DarkViolet", ColorCode = 0xFF9400D3},
         new ColorDescriptor{ColorName = "DeepPink", ColorCode = 0xFFFF1493},
         new ColorDescriptor{ColorName = "DeepSkyBlue", ColorCode = 0xFF00BFFF},
         new ColorDescriptor{ColorName = "DimGray", ColorCode = 0xFF696969},
         new ColorDescriptor{ColorName = "DodgerBlue", ColorCode = 0xFF1E90FF},
         new ColorDescriptor{ColorName = "Firebrick", ColorCode = 0xFFB22222},
         new ColorDescriptor{ColorName = "FloralWhite", ColorCode = 0xFFFFFAF0},
         new ColorDescriptor{ColorName = "ForestGreen", ColorCode = 0xFF228B22},
         new ColorDescriptor{ColorName = "Fuchsia", ColorCode = 0xFFFF00FF},
         new ColorDescriptor{ColorName = "Gainsboro", ColorCode = 0xFFDCDCDC},
         new ColorDescriptor{ColorName = "GhostWhite", ColorCode = 0xFFF8F8FF},
         new ColorDescriptor{ColorName = "Gold", ColorCode = 0xFFFFD700},
         new ColorDescriptor{ColorName = "Goldenrod", ColorCode = 0xFFDAA520},
         new ColorDescriptor{ColorName = "Gray", ColorCode = 0xFF808080},
         new ColorDescriptor{ColorName = "Green", ColorCode = 0xFF008000},
         new ColorDescriptor{ColorName = "GreenYellow", ColorCode = 0xFFADFF2F},
         new ColorDescriptor{ColorName = "Honeydew", ColorCode = 0xFFF0FFF0},
         new ColorDescriptor{ColorName = "HotPink", ColorCode = 0xFFFF69B4},
         new ColorDescriptor{ColorName = "IndianRed", ColorCode = 0xFFCD5C5C},
         new ColorDescriptor{ColorName = "Indigo", ColorCode = 0xFF4B0082},
         new ColorDescriptor{ColorName = "Ivory", ColorCode = 0xFFFFFFF0},
         new ColorDescriptor{ColorName = "Khaki", ColorCode = 0xFFF0E68C},
         new ColorDescriptor{ColorName = "Lavender", ColorCode = 0xFFE6E6FA},
         new ColorDescriptor{ColorName = "LavenderBlush", ColorCode = 0xFFFFF0F5},
         new ColorDescriptor{ColorName = "LawnGreen", ColorCode = 0xFF7CFC00},
         new ColorDescriptor{ColorName = "LemonChiffon", ColorCode = 0xFFFFFACD},
         new ColorDescriptor{ColorName = "LightBlue", ColorCode = 0xFFADD8E6},
         new ColorDescriptor{ColorName = "LightCoral", ColorCode = 0xFFF08080},
         new ColorDescriptor{ColorName = "LightCyan", ColorCode = 0xFFE0FFFF},
         new ColorDescriptor{ColorName = "LightGoldenrodYellow", ColorCode = 0xFFFAFAD2},
         new ColorDescriptor{ColorName = "LightGray", ColorCode = 0xFFD3D3D3},
         new ColorDescriptor{ColorName = "LightGreen", ColorCode = 0xFF90EE90},
         new ColorDescriptor{ColorName = "LightPink", ColorCode = 0xFFFFB6C1},
         new ColorDescriptor{ColorName = "LightSalmon", ColorCode = 0xFFFFA07A},
         new ColorDescriptor{ColorName = "LightSeaGreen", ColorCode = 0xFF20B2AA},
         new ColorDescriptor{ColorName = "LightSkyBlue", ColorCode = 0xFF87CEFA},
         new ColorDescriptor{ColorName = "LightSlateGray", ColorCode = 0xFF778899},
         new ColorDescriptor{ColorName = "LightSteelBlue", ColorCode = 0xFFB0C4DE},
         new ColorDescriptor{ColorName = "LightYellow", ColorCode = 0xFFFFFFE0},
         new ColorDescriptor{ColorName = "Lime", ColorCode = 0xFF00FF00},
         new ColorDescriptor{ColorName = "LimeGreen", ColorCode = 0xFF32CD32},
         new ColorDescriptor{ColorName = "Linen", ColorCode = 0xFFFAF0E6},
         new ColorDescriptor{ColorName = "Magenta", ColorCode = 0xFFFF00FF},
         new ColorDescriptor{ColorName = "Maroon", ColorCode = 0xFF800000},
         new ColorDescriptor{ColorName = "MediumAquamarine", ColorCode = 0xFF66CDAA},
         new ColorDescriptor{ColorName = "MediumBlue", ColorCode = 0xFF0000CD},
         new ColorDescriptor{ColorName = "MediumOrchid", ColorCode = 0xFFBA55D3},
         new ColorDescriptor{ColorName = "MediumPurple", ColorCode = 0xFF9370DB},
         new ColorDescriptor{ColorName = "MediumSeaGreen", ColorCode = 0xFF3CB371},
         new ColorDescriptor{ColorName = "MediumSlateBlue", ColorCode = 0xFF7B68EE},
         new ColorDescriptor{ColorName = "MediumSpringGreen", ColorCode = 0xFF00FA9A},
         new ColorDescriptor{ColorName = "MediumTurquoise", ColorCode = 0xFF48D1CC},
         new ColorDescriptor{ColorName = "MediumVioletRed", ColorCode = 0xFFC71585},
         new ColorDescriptor{ColorName = "MidnightBlue", ColorCode = 0xFF191970},
         new ColorDescriptor{ColorName = "MintCream", ColorCode = 0xFFF5FFFA},
         new ColorDescriptor{ColorName = "MistyRose", ColorCode = 0xFFFFE4E1},
         new ColorDescriptor{ColorName = "Moccasin", ColorCode = 0xFFFFE4B5},
         new ColorDescriptor{ColorName = "NavajoWhite", ColorCode = 0xFFFFDEAD},
         new ColorDescriptor{ColorName = "Navy", ColorCode = 0xFF000080},
         new ColorDescriptor{ColorName = "OldLace", ColorCode = 0xFFFDF5E6},
         new ColorDescriptor{ColorName = "Olive", ColorCode = 0xFF808000},
         new ColorDescriptor{ColorName = "OliveDrab", ColorCode = 0xFF6B8E23},
         new ColorDescriptor{ColorName = "Orange", ColorCode = 0xFFFFA500},
         new ColorDescriptor{ColorName = "OrangeRed", ColorCode = 0xFFFF4500},
         new ColorDescriptor{ColorName = "Orchid", ColorCode = 0xFFDA70D6},
         new ColorDescriptor{ColorName = "PaleGoldenrod", ColorCode = 0xFFEEE8AA},
         new ColorDescriptor{ColorName = "PaleGreen", ColorCode = 0xFF98FB98},
         new ColorDescriptor{ColorName = "PaleTurquoise", ColorCode = 0xFFAFEEEE},
         new ColorDescriptor{ColorName = "PaleVioletRed", ColorCode = 0xFFDB7093},
         new ColorDescriptor{ColorName = "PapayaWhip", ColorCode = 0xFFFFEFD5},
         new ColorDescriptor{ColorName = "PeachPuff", ColorCode = 0xFFFFDAB9},
         new ColorDescriptor{ColorName = "Peru", ColorCode = 0xFFCD853F},
         new ColorDescriptor{ColorName = "Pink", ColorCode = 0xFFFFC0CB},
         new ColorDescriptor{ColorName = "Plum", ColorCode = 0xFFDDA0DD},
         new ColorDescriptor{ColorName = "PowderBlue", ColorCode = 0xFFB0E0E6},
         new ColorDescriptor{ColorName = "Purple", ColorCode = 0xFF800080},
         new ColorDescriptor{ColorName = "Red", ColorCode = 0xFFFF0000},
         new ColorDescriptor{ColorName = "RosyBrown", ColorCode = 0xFFBC8F8F},
         new ColorDescriptor{ColorName = "RoyalBlue", ColorCode = 0xFF4169E1},
         new ColorDescriptor{ColorName = "SaddleBrown", ColorCode = 0xFF8B4513},
         new ColorDescriptor{ColorName = "Salmon", ColorCode = 0xFFFA8072},
         new ColorDescriptor{ColorName = "SandyBrown", ColorCode = 0xFFF4A460},
         new ColorDescriptor{ColorName = "SeaGreen", ColorCode = 0xFF2E8B57},
         new ColorDescriptor{ColorName = "SeaShell", ColorCode = 0xFFFFF5EE},
         new ColorDescriptor{ColorName = "Sienna", ColorCode = 0xFFA0522D},
         new ColorDescriptor{ColorName = "Silver", ColorCode = 0xFFC0C0C0},
         new ColorDescriptor{ColorName = "SkyBlue", ColorCode = 0xFF87CEEB},
         new ColorDescriptor{ColorName = "SlateBlue", ColorCode = 0xFF6A5ACD},
         new ColorDescriptor{ColorName = "SlateGray", ColorCode = 0xFF708090},
         new ColorDescriptor{ColorName = "Snow", ColorCode = 0xFFFFFAFA},
         new ColorDescriptor{ColorName = "SpringGreen", ColorCode = 0xFF00FF7F},
         new ColorDescriptor{ColorName = "SteelBlue", ColorCode = 0xFF4682B4},
         new ColorDescriptor{ColorName = "Tan", ColorCode = 0xFFD2B48C},
         new ColorDescriptor{ColorName = "Teal", ColorCode = 0xFF008080},
         new ColorDescriptor{ColorName = "Thistle", ColorCode = 0xFFD8BFD8},
         new ColorDescriptor{ColorName = "Tomato", ColorCode = 0xFFFF6347},
         new ColorDescriptor{ColorName = "Transparent", ColorCode = 0x00FFFFFF},
         new ColorDescriptor{ColorName = "Turquoise", ColorCode = 0xFF40E0D0},
         new ColorDescriptor{ColorName = "Violet", ColorCode = 0xFFEE82EE},
         new ColorDescriptor{ColorName = "Wheat", ColorCode = 0xFFF5DEB3},
         new ColorDescriptor{ColorName = "White", ColorCode = 0xFFFFFFFF},
         new ColorDescriptor{ColorName = "WhiteSmoke", ColorCode = 0xFFF5F5F5},
         new ColorDescriptor{ColorName = "Yellow", ColorCode = 0xFFFFFF00},
         new ColorDescriptor{ColorName = "YellowGreen", ColorCode = 0xFF9ACD32}
       };
        }
        /// <summary>
        /// Gets or sets the predefined colors.
        /// </summary>
        /// <value>The predefined colors.</value>
        public static List<ColorDescriptor> PredefinedColors { get; set; }

        /// <summary>
        /// Get a <see cref="ColorDescriptor"/> for a given color code.
        /// </summary>
        /// <param name="colorCode">The color code.</param>
        /// <returns></returns>
        public static ColorDescriptor ColorForValue(uint colorCode)
        {
            return (from c in PredefinedColors
                    where c.ColorCode == colorCode
                    select c).FirstOrDefault();
        }
    }
}
